@@ -154,7 +154,7 @@ make_par_values <- function(model,
     if (!is.na(distribution)) {
       message(
         paste(
-          "No specific parameters to alter values for specified.",
+          # "No specific parameters to alter values for specified.",
           "Altering all parameters."
         )
       )
@@ -174,34 +174,18 @@ make_par_values <- function(model,
 
     # warn if conditions are under-specified
     if ((length(x) != 1) && (length(commands) != length(names))) {
-      warning(
-        paste(
-          "A specified condition matches multiple parameters. In these",
-          "cases it is unclear which parameter value should be assigned to",
-          "which parameter. Assignment thus defaults to the order in which",
-          "parameters appear in 'parameters_df'. We advise checking that",
-          "parameter assignment was carried out as you intended. "
-        )
-      )
+      warning("Possible ambiguity: use additional arguments or check behavior in parameters_df.")
     }
 
 
     # forgive user when specifying across
     if ((length(names) > 0) && all(grepl("_", names))) {
-      if ((!is.na(statement) &&
-           (!grepl("param_set", statement) || !grepl("given", statement))) |
-           (all(is.na(param_set)) && all(is.na(given)))
-          ) {
-        warning(
-          paste(
-            "You are altering parameters on confounded nodes.",
-            "Alterations will be applied across all 'param_sets'.",
-            "If this is not the alteration behavior you intended,",
-            "try specifying the 'param_set' or 'given' option to more",
-            "clearly indicate parameters whose values you wish to alter."
-          )
-        )
-      }
+      # if ((!is.na(statement) &&
+      #      (!grepl("param_set", statement) || !grepl("given", statement))) |
+      #      (all(is.na(param_set)) && all(is.na(given)))
+      #     ) {
+      #   warning("Possible ambiguity: use additional arguments or check behavior in parameters_df.")
+      # }
 
       if (length(x) != length(names)) {
         x <- rep(x, each = length(names) / length(commands))
