@@ -13,23 +13,31 @@
 #'
 #' @export
 print.causal_model <- function(x, ...) {
+
+  nnt <- vapply(get_nodal_types(x), length, numeric(1), USE.NAMES = TRUE)
+
   cat("\nCausal statement: \n")
   cat(x$statement)
   cat("\n")
 
-  cat("\nNumber of types by node:\n")
-  print(vapply(get_nodal_types(x), length, numeric(1), USE.NAMES = TRUE))
+  cat("\nNumber of nodal types by node:\n")
+  print(nnt)
 
 
   if (!is.null(x$causal_types)) {
     cat("\nNumber of causal types:")
-    cat(paste0(" ", nrow(get_causal_types(x)), "\n"))
+    cat(paste0(" ", prod(nnt), "\n"))
   }
 
   if (!is.null(x$posterior_distribution) & !is.null(x$stan_objects)) {
     cat("\nModel has been updated and contains a posterior distribution with\n")
     cat(paste(x$stan_objects$stan_summary[[2]], "\n"))
     cat("Use inspect(model, 'stan_objects') to inspect stan summary\n\n")
+
+    if(x$stan_objects$stan_warnings != "") {
+      cat("Warnings passed from rstan during updating:\n")
+      cat(x$stan_objects$stan_warnings)
+    }
   }
 
   return(invisible(x))
@@ -95,11 +103,6 @@ summary.causal_model <- function(object, include = NULL, ...) {
     c(get_args_for(get_all_data_types, dots)) |>
     do.call(get_all_data_types, args = _)
 
-  object$prior_event_probabilities <-
-    list(object) |>
-    c(get_args_for(get_event_probabilities, dots)) |>
-    do.call(get_event_probabilities, args = _)
-
   object$parameter_names <-
     list(object) |>
     c(get_args_for(get_parameter_names, dots)) |>
@@ -136,6 +139,7 @@ summary.causal_model <- function(object, include = NULL, ...) {
         "posterior_event_probabilities",
         "data",
         "stanfit",
+        "stan_warnings",
         "stan_summary",
         "stan_xs"
       ))
@@ -147,6 +151,15 @@ summary.causal_model <- function(object, include = NULL, ...) {
       )
     }
 
+    # Larger objects generated on demand
+
+    if ("prior_event_probabilities" %in% include) {
+      object$prior_event_probabilities <-
+      list(object) |>
+      c(get_args_for(get_event_probabilities, dots)) |>
+      do.call(get_event_probabilities, args = _)
+    }
+
     if ("parameter_mapping" %in% include) {
     object$parameter_mapping <-
       list(object) |>
@@ -421,6 +434,7 @@ print.summary.causal_model <-
           "posterior_event_probabilities",
           "data",
           "stanfit",
+          "stan_warnings",
           "stan_summary",
           "stan_objects"
         ))
@@ -741,7 +755,7 @@ print.summary.causal_model <-
         }
       }
 
-      # stan_objects
+      # stan_objects: stanfit
       if ("stanfit" %in% what) {
         if (!is.null(x$stan_objects$stanfit)) {
           cat("\nstanfit\nStan model summary:\n")
@@ -752,6 +766,18 @@ print.summary.causal_model <-
         }
       }
 
+      # stan_objects: messages
+      if ("stan_warnings" %in% what) {
+        if (!is.null(x$stan_objects$stan_warnings)) {
+          cat("\nstan_warnings\nStan warnings generated during updating:\n")
+          cat(x$stan_objects$stan_warnings)
+        } else {
+          printout <- c(printout, "stan_warnings")
+          printout_upd <- c(printout_upd, "'keep_fit = TRUE'")
+        }
+      }
+
+
       # stan_objects
       if ("stan_summary" %in% what | "stan_objects" %in% what) {
         if (!is.null(x$stan_objects)) {
@@ -763,6 +789,8 @@ print.summary.causal_model <-
         }
       }
 
+
+
       if (length(printout) > 0) {
         if (length(printout_upd) > 0) {
           stop(
@@ -792,6 +820,13 @@ print.summary.causal_model <-
       cat("\n")
       cat("Note: To pose causal queries of this model use query_model()\n")
       cat("\n")
+
+      if(!is.null(x$posterior_distribution)){
+        if(x$stan_objects$stan_warnings != "") {
+          cat("Note: warnings passed from rstan during updating:\n")
+          cat(x$stan_objects$stan_warnings)
+        }}
+
     }
 
 
@@ -828,6 +863,13 @@ print.model_query <- function(x, ...) {
   case_level <- NULL
   given <- NULL
 
+  if(any("posteriors" %in% x$using)){
+    if(any(attr(x, "stan_warnings") != "")) {
+      cat("Note: warnings passed from rstan during updating:\n")
+      cat(attr(x, "stan_warnings"))
+      cat("\n")
+    }}
+
   text1 <- "Causal queries generated by query_model"
 
   # Output simplification
@@ -846,15 +888,41 @@ print.model_query <- function(x, ...) {
       text1 <- paste(text1, "(all at population level)")
     }
   }
+  if (all(c("query", "label", "given") %in% names(x))) {
+    if(all(x$query == x$label)) x <- select(x, -label)
+    if(all(x$label == paste(x$query, ":|:", x$given))) x <- dplyr::select(x, -query, -given)
+  }
 
+  if (all(c("query", "label") %in% names(x))  &  !("given" %in% names(x))) {
+    if(all(x$label == x$query)) x <- dplyr::select(x, -query, -given)
+  }
+
+  # Printout requires modification for markdown disambiguation
+  r <- x
   cat("\n")
   cat(text1)
-  print(knitr::kable(x, digits = 3))
+  r$label <- clean_text(r$label)
+  r$query <- clean_text(r$query)
+  r$given <- clean_text(r$given)
+  print(knitr::kable(r, digits = 3))
   cat("\n")
+
   return(invisible(x))
 }
 
 
+# Helper function to clean up the text
+clean_text <- function(column) {
+  if (!is.null(column)) {
+    column <- gsub("\\|", "OR", column)
+    column <- gsub(":OR:", " given ", column)
+    gsub("\\s+", " ", column)  # Return the cleaned column
+  } else {
+    NULL
+  }
+}
+
+
 
 
 #' Summarizing model queries
