@@ -78,6 +78,12 @@ set_confound <- function(model,
 
   is_a_model(model)
 
+  if (grepl("\\.", model$statement)) {
+    stop(
+      "Unsupported characters in variable names. No dots '.' in variable names for models with confounding."
+    )
+  }
+
   if (any(confound |> lapply(function(k)
     grepl(";", k)) |> unlist())) {
     stop("Please provide multipe confounds as a list")
@@ -214,7 +220,7 @@ set_confound <- function(model,
     model$prior_distribution <- NULL
   }
 
-  if (!is.null(model$posterior_distribution)) {
+  if (has_posterior(model)) {
     model$posterior_distribution <- NULL
   }
 
