@@ -348,7 +348,7 @@ set_restrictions <- function(model,
     model$prior_distribution <- NULL
   }
 
-  if (!is.null(model$posterior_distribution)) {
+  if (has_posterior(model)) {
     model$posterior_distribution <- NULL
   }
 
